#ifndef RZ_BUILD_VERSION_H
#define RZ_BUILD_VERSION_H

#define RZ_VERSION_MAJOR 0
#define RZ_VERSION_MINOR 6
#define RZ_VERSION_PATCH 3
#define RZ_VERSION_NUMBER 6003
#define RZ_VERSION "0.6.3"
/* #undef RZ_PACKAGER_VERSION */
/* #undef RZ_PACKAGER */
#endif
